package com.sun.java.swing.plaf.windows;

import java.awt.FontMetrics;
import sun.swing.SwingUtilities2;
import java.awt.Color;
import javax.swing.ButtonModel;
import javax.swing.JMenu;
import java.awt.Rectangle;
import java.awt.Graphics;
import javax.swing.plaf.ComponentUI;
import javax.swing.JComponent;
import javax.swing.JMenuItem;
import javax.swing.plaf.basic.BasicMenuItemUI;

public class WindowsMenuItemUI extends BasicMenuItemUI
{
    final WindowsMenuItemUIAccessor accessor;
    
    public WindowsMenuItemUI() {
        this.accessor = new WindowsMenuItemUIAccessor() {
            @Override
            public JMenuItem getMenuItem() {
                return WindowsMenuItemUI.this.menuItem;
            }
            
            @Override
            public TMSchema.State getState(JMenuItem menuItem) {
                return WindowsMenuItemUI.getState(this, menuItem);
            }
            
            @Override
            public TMSchema.Part getPart(JMenuItem menuItem) {
                return WindowsMenuItemUI.getPart(this, menuItem);
            }
        };
    }
    
    public static ComponentUI createUI(JComponent component) {
        return new WindowsMenuItemUI();
    }
    
    @Override
    protected void paintText(Graphics graphics, JMenuItem menuItem, Rectangle rectangle, String s) {
        if (isVistaPainting()) {
            paintText(this.accessor, graphics, menuItem, rectangle, s);
            return;
        }
        ButtonModel model = menuItem.getModel();
        Color color = graphics.getColor();
        if (model.isEnabled() && (model.isArmed() || (menuItem instanceof JMenu && model.isSelected()))) {
            graphics.setColor(this.selectionForeground);
        }
        WindowsGraphicsUtils.paintText(graphics, menuItem, rectangle, s, 0);
        graphics.setColor(color);
        return;
    }
    
    @Override
    protected void paintBackground(Graphics g, JMenuItem menuItem, Color bgColor) {
        if (isVistaPainting()) {
            paintBackground(this.accessor, g, menuItem, bgColor);
            return;
        }
        super.paintBackground(g, menuItem, bgColor);
        return;
    }
    
    static void paintBackground(WindowsMenuItemUIAccessor windowsMenuItemUIAccessor, Graphics graphics, JMenuItem menuItem, Color color) {
        XPStyle xp = XPStyle.getXP();
        assert !(!isVistaPainting(xp));
        if (isVistaPainting(xp)) {
            int width = menuItem.getWidth();
            int height = menuItem.getHeight();
            if (menuItem.isOpaque()) {
                Color color2 = graphics.getColor();
                graphics.setColor(menuItem.getBackground());
                graphics.fillRect(0, 0, width, height);
                graphics.setColor(color2);
            }
            xp.getSkin(menuItem, windowsMenuItemUIAccessor.getPart(menuItem)).paintSkin(graphics, 0, 0, width, height, windowsMenuItemUIAccessor.getState(menuItem));
        }
        return;
    }
    
    static void paintText(WindowsMenuItemUIAccessor windowsMenuItemUIAccessor, Graphics g, JMenuItem c, Rectangle rectangle, String s) {
        assert !(!isVistaPainting());
        if (isVistaPainting()) {
            TMSchema.State state = windowsMenuItemUIAccessor.getState(c);
            FontMetrics fontMetrics = SwingUtilities2.getFontMetrics(c, g);
            int displayedMnemonicIndex = c.getDisplayedMnemonicIndex();
            if (WindowsLookAndFeel.isMnemonicHidden()) {
                displayedMnemonicIndex = -1;
            }
            WindowsGraphicsUtils.paintXPText(c, windowsMenuItemUIAccessor.getPart(c), state, g, rectangle.x, rectangle.y + fontMetrics.getAscent(), s, displayedMnemonicIndex);
        }
        return;
    }
    
    static TMSchema.State getState(WindowsMenuItemUIAccessor windowsMenuItemUIAccessor, JMenuItem menuItem) {
        ButtonModel model = menuItem.getModel();
        if (model.isArmed()) {
            return model.isEnabled() ? TMSchema.State.HOT : TMSchema.State.DISABLEDHOT;
        }
        return model.isEnabled() ? TMSchema.State.NORMAL : TMSchema.State.DISABLED;
    }
    
    static TMSchema.Part getPart(WindowsMenuItemUIAccessor windowsMenuItemUIAccessor, JMenuItem menuItem) {
        return TMSchema.Part.MP_POPUPITEM;
    }
    
    static boolean isVistaPainting(XPStyle xpStyle) {
        return xpStyle != null && xpStyle.isSkinDefined(null, TMSchema.Part.MP_POPUPITEM);
    }
    
    static boolean isVistaPainting() {
        return isVistaPainting(XPStyle.getXP());
    }
}
